<?php

namespace Drupal\slick_browser\Plugin\EntityBrowser\FieldWidgetDisplay;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\entity_browser\FieldWidgetDisplayBase;
use Drupal\slick_browser\SlickBrowserDefault;
use Drupal\slick_browser\SlickBrowserUtil;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for Slick Browser entity display and or entity selection.
 */
abstract class SlickBrowserFieldWidgetDisplayBase extends FieldWidgetDisplayBase implements ContainerFactoryPluginInterface {

  /**
   * The blazy entity service.
   *
   * @var \Drupal\blazy\BlazyEntityInterface
   */
  protected $blazyEntity;


  /**
   * The blazy manager service.
   *
   * @var \Drupal\blazy\BlazyManagerInterface
   */
  protected $blazyManager;

  /**
   * The blazy oembed service.
   *
   * @var \Drupal\blazy\BlazyOEmbed
   */
  protected $blazyOembed;

  /**
   * The slick browser service.
   *
   * @var \Drupal\slick_browser\SlickBrowserInterface
   */
  protected $slickBrowser;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition
    );
    return self::injectServices($instance, $container);
  }

  /**
   * Injects DI services.
   */
  protected static function injectServices($instance, ContainerInterface $container) {
    $instance->slickBrowser = $container->get('slick_browser');
    $instance->blazyEntity  = isset($instance->blazyEntity) ? $instance->blazyEntity : $instance->slickBrowser->blazyEntity();
    $instance->blazyOembed  = isset($instance->blazyOembed) ? $instance->blazyOembed : $instance->slickBrowser->oembed();
    $instance->blazyManager = isset($instance->blazyManager) ? $instance->blazyManager : $instance->slickBrowser->blazyManager();

    return $instance;
  }

  /**
   * Returns the slick browser service.
   */
  public function slickBrowser() {
    return $this->slickBrowser;
  }

  /**
   * Returns the blazy entity service.
   */
  public function blazyEntity() {
    return $this->blazyEntity;
  }

  /**
   * Returns the blazy oEmbed service.
   */
  public function blazyOembed() {
    return $this->blazyOembed;
  }

  /**
   * Returns the blazy admin service.
   */
  public function blazyAdmin() {
    return $this->slickBrowser->blazyAdmin();
  }

  /**
   * Returns the blazy manager.
   */
  public function blazyManager() {
    return $this->blazyManager;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $is_widget = isset($form['#fields']);
    $settings = $this->buildSettings(SlickBrowserDefault::baseFieldWidgetDisplaySettings());

    // This form can be either on Entity browser display, or widget plugin.
    $element['_context'] = [
      '#type' => 'hidden',
      '#default_value' => $is_widget ? 'widget' : 'selection',
    ];

    // Do not use hook_field_widget_third_party_settings_form(), as this form is
    // also duplicated at "Manage form display" page.
    if ($is_widget) {
      $definition = $this->getScopedFormElements();
      $element += $this->blazyAdmin()->baseForm($definition);
      $this->slickBrowser->buildSettingsForm($element, $definition);

      if (!in_array($this->configuration['entity_type'], ['media', 'file'])) {
        unset($element['image_style']);
      }
    }
    else {
      // This is the selection previews normally tiny thumbnails or labels.
      $element['selection_position'] = [
        '#type'    => 'select',
        '#title'   => $this->t('Selection position'),
        '#options' => [
          'left'        => $this->t('Left'),
          'right'       => $this->t('Right'),
          'bottom'      => $this->t('Bottom'),
          'over-bottom' => $this->t('Overlay bottom'),
        ],
        '#default_value' => isset($settings['selection_position']) ? $settings['selection_position'] : 'over-bottom',
        '#description'   => $this->t('Left and Right positions are more suitable for large displays such as Modal. Overlay means fixed positioned over the stage rather than sharing space. They are affected by the Slick Browser: Tabs positioning. If Tabs is placed at Top or Bottom, Selection position can be placed on the Right or Left, otherwise resulting in too narrow form. Adjust it accordingly.'),
      ];
    }

    return $element;
  }

  /**
   * Gets EB widget settings.
   */
  public function buildSettings($defaults = []) {
    $defaults = $defaults ?: SlickBrowserDefault::widgetSettings();
    $settings = [];
    foreach ($defaults as $key => $default) {
      $settings[$key] = isset($this->configuration[$key]) ? $this->configuration[$key] : $default;
    }

    // Only load the Blazy library if using SB browsers, but not SB widgets.
    $settings['_detached']    = !empty($settings['style']);
    $settings['entity_type']  = isset($this->configuration['entity_type']) ? $this->configuration['entity_type'] : '';
    $settings['media_switch'] = empty($settings['media_switch']) ? 'media' : $settings['media_switch'];

    // Enforces thumbnail without video iframes for tiny selection thumbnail.
    $selection = $settings['_context'] == 'selection';
    if ($selection) {
      $settings['lazy'] = $settings['ratio'] = '';
      $settings['_noiframe'] = TRUE;
    }

    return array_merge(SlickBrowserDefault::entitySettings(), $settings);
  }

  /**
   * Defines the scope for the form elements.
   */
  public function getScopedFormElements() {
    return [
      'settings' => $this->buildSettings(),
      'target_type' => $this->configuration['entity_type'],
    ] + SlickBrowserUtil::scopedFormElements();
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $dependencies = parent::calculateDependencies();
    foreach (['image_style', 'thumbnail_style'] as $style) {
      if (isset($this->configuration[$style]) && $image_style = $this->blazyManager->getEntityTypeManager()->getStorage($style)->load($this->configuration[$style])) {
        $dependencies[$image_style->getConfigDependencyKey()][] = $image_style->getConfigDependencyName();
      }
    }
    return $dependencies;
  }

}
