<?php

namespace Drupal\slick_browser;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\entity_browser\Entity\EntityBrowser;

/**
 * Implements SlickBrowserWidgetInterface.
 */
class SlickBrowserWidget extends SlickBrowserAlter implements SlickBrowserWidgetInterface {

  /**
   * Implements hook_field_widget_WIDGET_TYPE_form_alter().
   */
  public function fieldWidgetEntiyBrowserFormAlter(&$element, FormStateInterface $form_state, $context) {
    $settings = $this->widgetSettings($element, $context);

    // Always assumes no "Display style" of SB widgets is enabled.
    $element['#slick_browser'] = [];
    if ($settings['plugin_id'] == 'entity_browser_entity_reference') {
      $this->widgetEntityBrowserEntityReferenceFormAlter($element, $form_state, $context, $settings);
    }
    elseif ($settings['plugin_id'] == 'entity_browser_file') {
      $this->widgetEntityBrowserFileFormAlter($element, $form_state, $context, $settings);
    }

    // Ony proceed if we are conciously allowed via "Display style" option.
    if (empty($element['#slick_browser'])) {
      return;
    }

    // We are here because we are allowed to.
    // Build common settings to all supported plugins.
    // The non-image field type has 'display_field', 'description'.
    $settings = array_merge($settings, $element['#slick_browser']);

    // Adds own theme to style entity browser elements.
    $this->widgetElement($element, $settings, $context);
  }

  /**
   * Returns the widget settings.
   */
  private function widgetSettings(array &$element, $context) {
    $field = $context['items']->getFieldDefinition();
    $widget_settings = $context['widget']->getSettings();
    $field_settings = $field->getSettings();

    $settings = [];
    foreach (['alt_field', 'title_field', 'target_type'] as $key) {
      $settings[$key] = isset($field_settings[$key]) ? $field_settings[$key] : FALSE;
    }

    $settings['plugin_id'] = $context['widget']->getPluginId();
    $settings['count'] = isset($element['current'], $element['current']['items']) ? count($element['current']['items']) : 0;
    $settings['cardinality'] = $field->getFieldStorageDefinition()->getCardinality();
    $settings['field_name'] = $context['items']->getName();
    $settings['field_type'] = $field->getType();
    $settings['entity_type_id'] = $field->getEntityTypeId();
    $settings['_eb'] = $settings['use_media'] = FALSE;

    if (!empty($widget_settings['preview_image_style']) && empty($settings['image_style'])) {
      $settings['image_style'] = $widget_settings['preview_image_style'];
    }

    if (in_array($settings['plugin_id'], ['entity_browser_entity_reference', 'entity_browser_file'])) {
      $settings['_eb'] = TRUE;
      $settings['_browser'] = TRUE;
    }

    // Chances are SB browsers within iframes/modals, even if no SB widgets.
    // Or using any of SB field_widget_display.
    // Only EntityReferenceBrowserWidget has field_widget_display, not FBW.
    $widget = empty($widget_settings['field_widget_display']) ? [] : $widget_settings['field_widget_display'];
    if ($widget && ($widget == 'slick_browser_file' || $widget == 'slick_browser_media')) {
      $settings['use_media'] = TRUE;
      $element['#attached']['library'][] = 'slick_browser/media';
    }

    // Load relevant assets based on the chosen SB browsers plugins.
    if (!empty($widget_settings['entity_browser'])) {
      $id = $widget_settings['entity_browser'];
      if ($eb = EntityBrowser::load($id)) {
        // Selection displays: modal, iframe, etc.
        if ($eb->getDisplay()->getPluginId() == 'modal') {
          $element['#attached']['library'][] = 'slick_browser/modal';
        }
        // @todo: Check for iframe, etc.
        // Entity display plugins: slick_browser_file, slick_browser_media, etc.
        // $config_display = $eb->getSelectionDisplay()->getConfiguration();
        // $config_widget = $eb->getWidgetSelector()->getConfiguration();
      }
    }

    return $settings;
  }

  /**
   * Modify the widget form element.
   */
  private function widgetElement(array &$element, array $settings, $context) {
    // Build the SB widgets, nothing to do with SB browsers here on.
    // This used to be "Slick Widget", moved into "Slick Browser".
    $element['#attributes']['class'][] = 'sb sb--details';
    if (empty($element['#open'])) {
      $element['#open'] = TRUE;
      $element['#attributes']['class'][] = 'sb--details-hidden';
    }

    if (in_array($settings['style'], ['grid', 'column', 'single'])) {
      $element['#attributes']['class'][] = 'sb--static';
    }
    $element['current']['#settings']       = $settings;
    $element['current']['#attributes']     = [];
    $element['current']['#theme_wrappers'] = [];
    $element['current']['#theme']          = 'slick_browser';

    $children = Element::children($element['current']);
    if ($settings['plugin_id'] == 'entity_browser_file' && $children) {
      unset($element['current']['#type'], $element['current']['#header'], $element['current']['#tabledrag']);
    }

    $this->widgetAttach($element, $settings);

    // Cleans up dup children if already merged into items property.
    if (isset($element['current']['items'])) {
      foreach ($children as $child) {
        if ($child == 'items') {
          continue;
        }
        unset($element['current'][$child]);
      }
    }
  }

  /**
   * Provides attachments.
   */
  private function widgetAttach(array &$element, array $settings) {
    // Enforce Blazy to work with hidden element such as with EB selection.
    // Prevents collapsed details from breaking lazyload.
    $load = $this->slickBrowser->blazyManager()->attach($settings);
    $load['drupalSettings']['blazy']['loadInvisible'] = TRUE;
    $load['library'][] = 'slick_browser/widget';
    $load['library'][] = 'slick_browser/tabs';

    // Disable tabledrag, including FBW table CSS, for Slick/ CSS grid.
    if ($settings['plugin_id'] == 'entity_browser_file') {
      $attachments = $load;
    }
    else {
      $attachments = isset($element['current'], $element['current']['#attached']) ? NestedArray::mergeDeep($element['current']['#attached'], $load) : $load;
    }

    $element['current']['#attached'] = $attachments;
  }

  /**
   * Implements hook_field_widget_WIDGET_TYPE_form_alter().
   */
  private function widgetEntityBrowserEntityReferenceFormAlter(&$element, FormStateInterface $form_state, $context, $config) {
    $widget_settings = $context['widget']->getSettings();
    if (empty($widget_settings['field_widget_display']) || strpos($widget_settings['field_widget_display'], 'slick_browser') === FALSE) {
      return;
    }

    $settings = $widget_settings['field_widget_display_settings'] + $config;
    if (empty($settings['style'])) {
      return;
    }

    $settings['plugin_id_sb'] = $widget_settings['field_widget_display'];
    $element['#slick_browser'] = array_merge(SlickBrowserDefault::entitySettings(), $settings);

    // Yet only proceed if entities are given, or at least one upload.
    $this->widgetDisplay($element, $settings, $context);
  }

  /**
   * Implements hook_field_widget_WIDGET_TYPE_form_alter().
   */
  private function widgetEntityBrowserFileFormAlter(&$element, FormStateInterface $form_state, $context, $config) {
    $widget_settings = $context['widget']->getSettings();

    if (empty($widget_settings['entity_browser']) || strpos($widget_settings['entity_browser'], 'slick_browser') === FALSE) {
      return;
    }

    $settings = SlickBrowserUtil::buildThirdPartySettings($context['widget']) + $config;
    if (empty($settings['style'])) {
      return;
    }

    // Allows Slick Browser to remove File Browser empty table.
    $element['#slick_browser'] = $settings;

    // Yet only proceed if entities are given, or at least one upload.
    $this->widgetDisplay($element, $settings, $context);
  }

  /**
   * Prepare entity displays.
   */
  private function widgetDisplay(array &$element, array $settings, $context) {
    // Yet only proceed if entities are given.
    $entities = isset($element['entity_browser']) ? $element['entity_browser']['#default_value'] : [];
    if (empty($entities)) {
      // Hence we have single entity, or cardinality 1.
      $children = Element::children($element['current']);
      foreach ($children as $item_id) {
        // Image item.
        if (isset($element['current'][$item_id]['display'], $element['current'][$item_id]['display']['#style_name']) && !empty($settings['image_style'])) {
          $element['current'][$item_id]['display']['#style_name'] = $settings['image_style'];
        }
        /** @var \Drupal\file\Plugin\Field\FieldType\FileFieldItemList **/
        elseif (isset($context['items']) && ($items = $context['items']) && !$items->isEmpty()) {
          $entity = $items->getEntity();
          $settings['delta'] = 0;

          $this->widgetItemDisplay($element, $settings, $entity, $item_id);
        }
      }
    }
    else {
      // Hence we have multiple entities.
      foreach ($entities as $delta => $entity) {
        $settings['delta'] = $delta;
        $this->widgetItemDisplay($element, $settings, $entity);
      }
    }
  }

  /**
   * Prepares entity item display.
   */
  private function widgetItemDisplay(array &$element, array $settings, $entity, $item_id = NULL) {
    /** @var \Drupal\file\Entity\File $entity */
    // @todo: Only provides custom image, if no URI available.
    $data = $this->slickBrowser->blazyEntity()->oembed()->getImageItem($entity);
    $data['settings'] = isset($data['settings']) ? array_merge($settings, $data['settings']) : $settings;

    $display = $this->slickBrowser->blazyEntity()->build($data, $entity, $entity->label());
    $settings = $data['settings'];
    $settings['bundle'] = empty($settings['bundle']) ? $entity->bundle() : $settings['bundle'];

    $display['#settings'] = isset($display['#settings']) ? array_merge($settings, $display['#settings']) : $settings;

    $element['current'][$item_id ?: $entity->id()]['display'] = $display;
  }

}
